@@ -26,7 +26,7 @@ use lsp_types::{
 };
 use project_model::TargetKind;
 use serde_json::json;
-use stdx::format_to;
+use stdx::{format_to, never};
 use syntax::{algo, ast, AstNode, TextRange, TextSize};
 
 use crate::{
@@ -233,12 +233,15 @@ pub(crate) fn handle_join_lines(
     params: lsp_ext::JoinLinesParams,
 ) -> Result<Vec<lsp_types::TextEdit>> {
     let _p = profile::span("handle_join_lines");
+
+    let config = snap.config.join_lines();
     let file_id = from_proto::file_id(&snap, &params.text_document.uri)?;
     let line_index = snap.file_line_index(file_id)?;
+
     let mut res = TextEdit::default();
     for range in params.ranges {
         let range = from_proto::text_range(&line_index, range);
-        let edit = snap.analysis.join_lines(FileRange { file_id, range })?;
+        let edit = snap.analysis.join_lines(&config, FileRange { file_id, range })?;
         match res.union(edit) {
             Ok(()) => (),
             Err(_edit) => {
@@ -246,8 +249,8 @@ pub(crate) fn handle_join_lines(
             }
         }
     }
-    let res = to_proto::text_edit_vec(&line_index, res);
-    Ok(res)
+
+    Ok(to_proto::text_edit_vec(&line_index, res))
 }
 
 pub(crate) fn handle_on_enter(
@@ -868,7 +871,7 @@ pub(crate) fn handle_hover(
 ) -> Result<Option<lsp_ext::Hover>> {
     let _p = profile::span("handle_hover");
     let position = from_proto::file_position(&snap, params.text_document_position_params)?;
-    let info = match snap.analysis.hover(position, &snap.config.hover())? {
+    let info = match snap.analysis.hover(&snap.config.hover(), position)? {
         None => return Ok(None),
         Some(info) => info,
     };
@@ -1130,41 +1133,51 @@ pub(crate) fn handle_code_lens(
     let file_id = from_proto::file_id(&snap, &params.text_document.uri)?;
     let cargo_target_spec = CargoTargetSpec::for_file(&snap, file_id)?;
 
-    let lenses = snap
-        .analysis
-        .annotations(
-            file_id,
-            AnnotationConfig {
-                binary_target: cargo_target_spec
-                    .map(|spec| {
-                        matches!(
-                            spec.target_kind,
-                            TargetKind::Bin | TargetKind::Example | TargetKind::Test
-                        )
-                    })
-                    .unwrap_or(false),
-                annotate_runnables: lens_config.runnable(),
-                annotate_impls: lens_config.implementations,
-                annotate_references: lens_config.refs,
-                annotate_method_references: lens_config.method_refs,
-                run: lens_config.run,
-                debug: lens_config.debug,
-            },
-        )?
-        .into_iter()
-        .map(|annotation| to_proto::code_lens(&snap, annotation).unwrap())
-        .collect();
+    let annotations = snap.analysis.annotations(
+        &AnnotationConfig {
+            binary_target: cargo_target_spec
+                .map(|spec| {
+                    matches!(
+                        spec.target_kind,
+                        TargetKind::Bin | TargetKind::Example | TargetKind::Test
+                    )
+                })
+                .unwrap_or(false),
+            annotate_runnables: lens_config.runnable(),
+            annotate_impls: lens_config.implementations,
+            annotate_references: lens_config.refs,
+            annotate_method_references: lens_config.method_refs,
+        },
+        file_id,
+    )?;
 
-    Ok(Some(lenses))
+    let mut res = Vec::new();
+    for a in annotations {
+        to_proto::code_lens(&mut res, &snap, a)?;
+    }
+
+    Ok(Some(res))
 }
 
 pub(crate) fn handle_code_lens_resolve(
     snap: GlobalStateSnapshot,
     code_lens: CodeLens,
 ) -> Result<CodeLens> {
-    let annotation = from_proto::annotation(&snap, code_lens)?;
+    let annotation = from_proto::annotation(&snap, code_lens.clone())?;
+    let annotation = snap.analysis.resolve_annotation(annotation)?;
 
-    to_proto::code_lens(&snap, snap.analysis.resolve_annotation(annotation)?)
+    let mut acc = Vec::new();
+    to_proto::code_lens(&mut acc, &snap, annotation)?;
+
+    let res = match acc.pop() {
+        Some(it) if acc.is_empty() => it,
+        _ => {
+            never!();
+            code_lens
+        }
+    };
+
+    Ok(res)
 }
 
 pub(crate) fn handle_document_highlight(
@@ -1250,7 +1263,7 @@ pub(crate) fn handle_inlay_hints(
     let line_index = snap.file_line_index(file_id)?;
     Ok(snap
         .analysis
-        .inlay_hints(file_id, &snap.config.inlay_hints())?
+        .inlay_hints(&snap.config.inlay_hints(), file_id)?
         .into_iter()
         .map(|it| to_proto::inlay_hint(&line_index, it))
         .collect())
@@ -1539,12 +1552,12 @@ fn runnable_action_links(
         return None;
     }
 
-    let action: &'static _ = runnable.action();
+    let title = runnable.title();
     to_proto::runnable(snap, runnable).ok().map(|r| {
         let mut group = lsp_ext::CommandLinkGroup::default();
 
         if hover_actions_config.run {
-            let run_command = to_proto::command::run_single(&r, action.run_title);
+            let run_command = to_proto::command::run_single(&r, &title);
             group.commands.push(to_command_link(run_command, r.label.clone()));
         }
 
