@@ -988,25 +988,42 @@ pub(crate) fn runnable(
 }
 
 pub(crate) fn code_lens(
+    acc: &mut Vec<lsp_types::CodeLens>,
     snap: &GlobalStateSnapshot,
     annotation: Annotation,
-) -> Result<lsp_types::CodeLens> {
+) -> Result<()> {
     match annotation.kind {
-        AnnotationKind::Runnable { debug, runnable: run } => {
+        AnnotationKind::Runnable(run) => {
             let line_index = snap.file_line_index(run.nav.file_id)?;
             let annotation_range = range(&line_index, annotation.range);
 
-            let action = run.action();
+            let title = run.title();
+            let can_debug = match run.kind {
+                ide::RunnableKind::DocTest { .. } => false,
+                ide::RunnableKind::TestMod { .. }
+                | ide::RunnableKind::Test { .. }
+                | ide::RunnableKind::Bench { .. }
+                | ide::RunnableKind::Bin => true,
+            };
             let r = runnable(snap, run)?;
 
-            let command = if debug {
-                command::debug_single(&r)
-            } else {
-                let title = action.run_title.to_string();
-                command::run_single(&r, &title)
-            };
-
-            Ok(lsp_types::CodeLens { range: annotation_range, command: Some(command), data: None })
+            let lens_config = snap.config.lens();
+            if lens_config.run {
+                let command = command::run_single(&r, &title);
+                acc.push(lsp_types::CodeLens {
+                    range: annotation_range,
+                    command: Some(command),
+                    data: None,
+                })
+            }
+            if lens_config.debug && can_debug {
+                let command = command::debug_single(&r);
+                acc.push(lsp_types::CodeLens {
+                    range: annotation_range,
+                    command: Some(command),
+                    data: None,
+                })
+            }
         }
         AnnotationKind::HasImpls { position: file_position, data } => {
             let line_index = snap.file_line_index(file_position.file_id)?;
@@ -1045,7 +1062,7 @@ pub(crate) fn code_lens(
                 )
             });
 
-            Ok(lsp_types::CodeLens {
+            acc.push(lsp_types::CodeLens {
                 range: annotation_range,
                 command,
                 data: Some(to_value(lsp_ext::CodeLensResolveData::Impls(goto_params)).unwrap()),
@@ -1074,13 +1091,14 @@ pub(crate) fn code_lens(
                 )
             });
 
-            Ok(lsp_types::CodeLens {
+            acc.push(lsp_types::CodeLens {
                 range: annotation_range,
                 command,
                 data: Some(to_value(lsp_ext::CodeLensResolveData::References(doc_pos)).unwrap()),
             })
         }
     }
+    Ok(())
 }
 
 pub(crate) mod command {
